/*
 * Config.cpp
 *
 * Copyright Notice: see copyright.txt
 *
 * Date: 3/8/2012 (D/M/Y)
 * Author: Dmitrij Lioubartsev
 * Email: dmitrijl42@gmail.com
 *
 * For config file specifications of parameters and syntax, see readme.txt.
 */

#include "Config.h"
#include <fstream>
#include <iostream>
#include <cstdlib>
using namespace std;

#define WHITE_SPACE(x) ((x == ' ') || (x == '\t') || (x == '\n')|| (x == '=') || (x == ':') || (x == '#') || (x == '\r'))

Config::Config() {
	//set default values
	output = "default";
	xmol = false;
	r_min = 20;
	// ratios.insert(std::pair<string,double>("Si",1.0)); <- this is empty as well.
	// eliminate.push_back("nothing"); <- this should be empty
	ff_file = "default";
	charge_tot = 0;
	car = false;

	//other
	fixCharge = false;
}

/*
 * Read the specified config file. See readme.txt fore more info
 * about the file syntax.
 */
bool Config::readFile(const char * configFile) {
	ifstream conFile(configFile);
	if(conFile) { //if file exists
		while(!conFile.eof()) {
			if(skipWhiteSpace(conFile)) //skip whitespaces
				break;

			string token("");
			if(getWord(conFile,token))  //get word
				break;

			if(skipWhiteSpace(conFile)) //skip whitespaces after
				break;

			string value("");
			getWord(conFile,value); //get value

			if(token == "output") {
				output = value;
			} else if(token == "make_xmol") {
				xmol = atoi(value.c_str());
			} else if(token == "R_min" || token == "r_min") {
				r_min = atof(value.c_str());
			} else if(token == "substitute" || token == "substitution") {
				for(int i = 0; i < atoi(value.c_str()); i++) {
					if(skipWhiteSpace(conFile)) //skip whitespaces
						break;
					string name("");
					if(getWord(conFile,name)) //get element
						break;
					if(skipWhiteSpace(conFile)) //skip whitespaces
						break;
					string ratioStr("");
					getWord(conFile,ratioStr); //get its ratio
					double ratio = atof(ratioStr.c_str());
					ratios.push_back(pair<string,double>(name,ratio));
				}
			} else if(token == "exclude" || token == "exclusion") {
				for(int i = 0; i < atoi(value.c_str()); i++) {
					if(skipWhiteSpace(conFile)) //skip whitespaces
						break;
					string formula("");
					getWord(conFile,formula);
					exclude.push_back(formula);
				}
			} else if(token == "force_field") {
				ff_file = value;
			} else if(token == "charge_tot") {
				fixCharge = true;
				charge_tot = atof(value.c_str());
			} else if(token == "make_car") {
				car = atoi(value.c_str());
			} //else do nothing
		}
		return true;
	} else {
		return false;
	}
}

Config::~Config() {
	// do nothing
}

/*
 * Skips all whitespace characters.
 * Return true if eof reached.
 */
bool Config::skipWhiteSpace(ifstream& stream) {
	char c = ' ';

	bool eof = false;
	while(WHITE_SPACE(c)) {
		stream.get(c);
		if(stream.eof()) {
			eof = true;
			break; //reached end of file, stop
		}
		if(c == '#')  {//comment
			stream.ignore(256, '\n');
			if(stream.eof()) {
				eof = true;
				break;
			}
		}
	}
	stream.unget();
	return eof;
}

/*
 * Get the next word, and place it in parameter word.
 * Returns true in case of eof.
 */
bool Config::getWord(ifstream& stream, string& word) {
	word = "";
	char c;
	bool eof = false;

	while(!(WHITE_SPACE(c))) {
		stream.get(c);
		if(stream.eof()) {
			eof = true;
			break; //reached end of file, stop
		}
		word.append(1,c);
	}
	if(!eof) {
		stream.unget();
		word.erase(word.length()-1,1);
	}
	return eof;
}

/*
 * Creates a configuration file with the current values.
 */
void Config::createConf() {
	ofstream conFile("input.cfg");

	//first create some explanation comments
	conFile << "##################################################################" << endl
			<< "# This is an auto-generated configration file with default values." << endl
			<< "# This is generated when no input configuration file is specified, and" << endl
			<< "# the default \"input.cfg\" does not exist." << endl
			<< "# For file syntax and parameters, see readme.txt" << endl
			<< "##################################################################" << endl << endl << endl;

	//now put values
	conFile << "output = " << output << endl;
	conFile << "make_xmol = " << xmol << endl;
	conFile << "make_car = " << car << endl;
	conFile << "R_min = " << r_min << endl;
	conFile << "substitute = " << ratios.size() << endl;
	vector<pair<string,double> >::iterator it;
	for(it = ratios.begin(); it != ratios.end();it++) {
		conFile << "   " << (it->first) << " : " << (it->second) << endl;
	}
	conFile << "exclude = " << exclude.size() << endl;
	vector<string>::iterator iter;
	for(iter = exclude.begin(); iter != exclude.end();iter++) {
		conFile << "   " << (*iter) << endl;
	}
	conFile << "force_field = " << ff_file << endl;
	conFile << "# charge_tot = " << charge_tot << endl;

	conFile.close();
}



#undef WHITE_SPACE

