/*
 * cifo.cpp
 *
 * Copyright Notice: see copyright.txt
 *
 * Date: 3/8/2012 (D/M/Y)
 * Author: Dmitrij Lioubartsev
 * Email: dmitrijl42@gmail.com
 *
 * Main source file. Creates the classes needed for the task
 * and calls the proper methods with some error testing.
 */

#include <string>
#include <iostream>
#include <cctype>
#include <cstring>
#include <vector>
using namespace std;

#include "CifCrystal.h"
#include "CifParser.h"
#include "Config.h"
#include "OutputWriter.h"
#include "Substitution.h"

void removeFileExtension(string& fileName);

int main(int argc, const char* argv[]) {

	CifCrystal crystal = CifCrystal(); // The crystal to work with
	CifParser parser(&crystal);	//.cif file reader
	bool stop = false;

	if(argc == 1) { // no arguments given
		stop = true;
		cerr << "No input parameters given." << endl;
	}

	Config cfg;

	if(!stop) {
	if(argc > 2) {  //config file specified
		if(! cfg.readFile(argv[2])) {
			cout << "WARNING: specified config file does not exist. "
					"Using default values." << endl;
		}
	}
	else {
		//config file not specified, use default name
		if(!cfg.readFile()) {
			cout << "Note: no configuration file specified, but"
					" the default \"input.cfg\" does not exist." << endl
					<< "Using default values and creating config "
					"file with default values." << endl;
			cfg.createConf();
		}

	}
	}

	if(!stop)
	if(!parser.opencif(argv[1])) {
		stop = true;
		cerr << "File " << argv[1] << " does not exist." << endl;
	}

	if(!stop) {
		parser.readcif();
	bool * areSet = crystal.areSet();

	if(!areSet[0]) {
		stop = true;
		cerr << "Could not find value for _cell_length_a "
				"in the file " << argv[1] << ". Stopping program." << endl;
	}
	if(!areSet[1]) {
		stop = true;
		cerr << "Could not find value for _cell_length_b "
				"in the file " << argv[1] << ". Stopping program."<< endl;
	}
	if(!areSet[2]) {
		stop = true;
		cerr << "Could not find value for _cell_length_c "
				"in the file " << argv[1] << ". Stopping program."<< endl;
	}
	if(!areSet[3]) {
		stop = true;
		cerr << "Could not find value for _cell_angle_alpha "
				"in the file " << argv[1] << ". Stopping program."<< endl;
	}
	if(!areSet[4]) {
		stop = true;
		cerr << "Could not find value for _cell_angle_beta "
				"in the file " << argv[1] << ". Stopping program."<< endl;
	}
	if(!areSet[5]) {
		stop = true;
		cerr << "Could not find value for _cell_angle_gamma "
				"in the file " << argv[1] << ". Stopping program."<< endl;
	}
	if(crystal.numAtoms() == 0) {
		stop = true;
		cerr << "Could not find any data about atoms in the file "
				<< argv[1] << ". Stopping program."<< endl;
	}
	if(!crystal.okData()) {
		stop = true;
		cerr << "Atom data in the file " << argv[1] << " is incomplete, "
				"one or more atoms are missing some part of information "
				"about them. Stopping program." << endl;
	}

	}

	if(!stop)
		crystal.processSymmetries();

	if(!stop) {
		crystal.makeSuperCell(cfg.getR_min());
		cout << "Supercell formed:\t" << crystal.getDupa() << "a\t"
				<< crystal.getDupb() << "b\t" << crystal.getDupc()
				<< 'c' << endl;
		cout.precision(2);
		cout.setf(ios_base::fixed);
		cout << "Final sizes:     \t"
				<< crystal.getNewa() << "   " << crystal.getNewb()
				<< "   " << crystal.getNewc() << endl;
		cout << "With angles:     \t" <<  crystal.get_alpha() << "   "
				<< crystal.get_beta() << "   " << crystal.get_gamma()
				<< endl;
		cout.precision(0);

		crystal.makeTransformations();
	}

	string output("");

	if(!stop) {
		if(cfg.getOutput() == "default") {
			output = argv[1];
		} else {
			output = cfg.getOutput();
		}
	}

	OutputWriter writer(output);

	if(!stop)
		crystal.makeBonds();

	if(!stop) {
		Substitution sub(crystal.getAtomList(), cfg.getSubRatios(), cfg.getElims());
		if(sub.substitute()) {
			cout << "Total number of atoms: " << crystal.numAtoms() << endl;
			sub.printAtomsSubstituted();
		}
		else
			stop = true;
	}

	if(!stop) {
		ForceField ff(crystal.getAtomList());
		string ff_file = cfg.getFF_file();
		if(ff_file == "default") {
			ff_file = argv[1];
			removeFileExtension(ff_file);
			ff_file.append(".ff");
		}
		if(!ff.loadFromFile(ff_file)) {
			cout << "WARNING: Could not find Force Field File "
					<< ff_file << "." << endl;
		}
		if(cfg.getFixCharge())
			ff.setCharge(cfg.getCharge_tot(), crystal.getAtomList(), true);
		else
			ff.setCharge(cfg.getCharge_tot(), crystal.getAtomList(), false);
		writer.createSmol(crystal);
	}

	if(!stop && cfg.getcar())
		writer.createCar(crystal, argv[1]);

	if(!stop && cfg.getxmol())
		writer.createXmol(crystal);


	return 0;
}

/*
 * removes the file extension from the string fileName
 */
void removeFileExtension(string& fileName) {
	unsigned int i = fileName.length()-1;
	while(fileName[i] != '.' && i > 0) {
		i--;
	}

	if(i != 0) {
		fileName.erase(i,fileName.length()-i);
	}
}
